@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage
 
-  Copyright (c) 2007-2016 Yoran Heling
+  Copyright (c) 2007-2018 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -37,7 +37,8 @@
 
 
 static struct dir *root, *nextsel, *curdir;
-static char noconfirm = 0, ignoreerr = 0, state, seloption;
+static char noconfirm = 0, ignoreerr = 0, state;
+static signed char seloption;
 static int lasterrno;
 
 
@@ -70,7 +71,9 @@ static void delete_draw_progress() {
   nccreate(6, 60, "Deleting...");
 
   ncaddstr(1, 2, cropstr(getpath(curdir), 47));
-  ncaddstr(4, 41, "Press q to abort");
+  ncaddstr(4, 41, "Press ");
+  addchc(UIC_KEY, 'q');
+  addstrc(UIC_DEFAULT, " to abort");
 }
 
 
