@@ -1,6 +1,6 @@
 /* ncdu - NCurses Disk Usage
 
-  Copyright (c) 2007-2016 Yoran Heling
+  Copyright (c) 2007-2018 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -92,24 +92,6 @@ static void hlink_check(struct dir *d) {
 }
 
 
-/* Make a copy of *item so that we'll keep it in memory. In the special case
- * of !root && orig, we need to copy over the name of *orig instead of *item.
- */
-static struct dir *item_copy(struct dir *item) {
-  struct dir *t;
-
-  if(!root && orig) {
-    t = malloc(SDIRSIZE+strlen(orig->name));
-    memcpy(t, item, SDIRSIZE);
-    strcpy(t->name, orig->name);
-  } else {
-    t = malloc(SDIRSIZE+strlen(item->name));
-    memcpy(t, item, SDIRSIZE+strlen(item->name));
-  }
-  return t;
-}
-
-
 /* Add item to the correct place in the memory structure */
 static void item_add(struct dir *item) {
   if(!root) {
@@ -129,16 +111,26 @@ static void item_add(struct dir *item) {
 }
 
 
-static int item(struct dir *item) {
-  struct dir *t;
+static int item(struct dir *dir, const char *name, struct dir_ext *ext) {
+  struct dir *t, *item;
 
   /* Go back to parent dir */
-  if(!item) {
+  if(!dir) {
     curdir = curdir->parent;
     return 0;
   }
 
-  item = item_copy(item);
+  if(!root && orig)
+    name = orig->name;
+
+  if(!extended_info)
+    dir->flags &= ~FF_EXT;
+  item = malloc(dir->flags & FF_EXT ? dir_ext_memsize(name) : dir_memsize(name));
+  memcpy(item, dir, offsetof(struct dir, name));
+  strcpy(item->name, name);
+  if(dir->flags & FF_EXT)
+    memcpy(dir_ext_ptr(item), ext, sizeof(struct dir_ext));
+
   item_add(item);
 
   /* Ensure that any next items will go to this directory */
